@@ -3,11 +3,12 @@
 
 #include "multitaskview.h"
 
-#include "config/treelandconfig.h"
 #include "output/output.h"
 #include "seat/helper.h"
 #include "surface/surfacecontainer.h"
 #include "workspace/workspace.h"
+#include "treelandconfig.hpp"
+#include "common/treelandlogging.h"
 
 #include <woutputitem.h>
 #include <woutputrenderwindow.h>
@@ -51,7 +52,7 @@ void Multitaskview::setActiveReason(ActiveReason activeReason)
 
 void Multitaskview::exit(SurfaceWrapper *surface, bool immediately)
 {
-    TreelandConfig::ref().setBlockActivateSurface(false);
+    Helper::instance()->setBlockActivateSurface(false);
 
     if (surface) {
         Helper::instance()->forceActivateSurface(surface);
@@ -182,7 +183,7 @@ QHash<int, QByteArray> MultitaskviewSurfaceModel::roleNames() const
     };
 }
 
-QModelIndex MultitaskviewSurfaceModel::index(int row, int column, const QModelIndex &parent) const
+QModelIndex MultitaskviewSurfaceModel::index(int row, int column, [[maybe_unused]] const QModelIndex &parent) const
 {
     if (row < 0 || row >= m_data.size())
         return QModelIndex();
@@ -218,11 +219,13 @@ void MultitaskviewSurfaceModel::updateZOrder()
     Q_EMIT dataChanged(index(0), index(rowCount() - 1), { ZOrderRole });
 }
 
-int MultitaskviewSurfaceModel::prevSameAppIndex(int index)
+uint MultitaskviewSurfaceModel::prevSameAppIndex(uint index)
 {
-    if (index < 0 || index >= count())
-        return -1;
-    auto circularPrev = [this](int i) {
+    if (index >= count()) {
+        qCWarning(treelandPlugin) << "prevSameAppIndex: invalid index" << index << "count:" << count();
+        return index;
+    }
+    auto circularPrev = [this](uint i) {
         return (i + count() - 1) % count();
     };
     auto i = circularPrev(index);
@@ -235,11 +238,13 @@ int MultitaskviewSurfaceModel::prevSameAppIndex(int index)
     return i;
 }
 
-int MultitaskviewSurfaceModel::nextSameAppIndex(int index)
+uint MultitaskviewSurfaceModel::nextSameAppIndex(uint index)
 {
-    if (index < 0 || index >= count())
-        return -1;
-    auto circularNext = [this](int i) {
+    if (index >= count()) {
+        qCWarning(treelandPlugin) << "nextSameAppIndex: invalid index" << index << "count:" << count();
+        return index;
+    }
+    auto circularNext = [this](uint i) {
         return (i + 1) % count();
     };
     auto i = circularNext(index);
@@ -263,7 +268,7 @@ void MultitaskviewSurfaceModel::setLayoutArea(const QRectF &newLayoutArea)
         return;
     m_layoutArea = newLayoutArea;
     initializeModel();
-    emit layoutAreaChanged();
+    Q_EMIT layoutAreaChanged();
 }
 
 bool MultitaskviewSurfaceModel::tryLayout(const QList<ModelDataPtr> &rawData,
@@ -274,12 +279,12 @@ bool MultitaskviewSurfaceModel::tryLayout(const QList<ModelDataPtr> &rawData,
     qreal acc = 0;
     auto devicePixelRatio = output()->outputItem()->devicePixelRatio();
     auto topContentMargin =
-        TreelandConfig::ref().multitaskviewTopContentMargin() / devicePixelRatio;
+        Helper::instance()->config()->multitaskviewTopContentMargin() / devicePixelRatio;
     auto bottomContentMargin =
-        TreelandConfig::ref().multitaskviewBottomContentMargin() / devicePixelRatio;
-    auto cellPadding = TreelandConfig::ref().multitaskviewCellPadding() / devicePixelRatio;
+        Helper::instance()->config()->multitaskviewBottomContentMargin() / devicePixelRatio;
+    auto cellPadding = Helper::instance()->config()->multitaskviewCellPadding() / devicePixelRatio;
     auto horizontalMargin =
-        TreelandConfig::ref().multitaskviewHorizontalMargin() / devicePixelRatio;
+        Helper::instance()->config()->multitaskviewHorizontalMargin() / devicePixelRatio;
     auto availWidth = std::max(0.0, layoutArea().width() - 2 * horizontalMargin);
     auto availHeight =
         std::max(0.0, layoutArea().height() - topContentMargin - bottomContentMargin);
@@ -299,7 +304,7 @@ bool MultitaskviewSurfaceModel::tryLayout(const QList<ModelDataPtr> &rawData,
         if (newAcc <= availWidth) {
             acc = newAcc;
             currow.append(modelData);
-        } else if (newAcc / availWidth > TreelandConfig::ref().multitaskviewLoadFactor()) {
+        } else if (newAcc / availWidth > Helper::instance()->config()->multitaskviewLoadFactor()) {
             acc = curW;
             nrows++;
             rowstmp.append(currow);
@@ -327,12 +332,12 @@ void MultitaskviewSurfaceModel::calcDisplayPos(const QList<ModelDataPtr> &rawDat
 {
     auto devicePixelRatio = output()->outputItem()->devicePixelRatio();
     auto topContentMargin =
-        TreelandConfig::ref().multitaskviewTopContentMargin() / devicePixelRatio;
+        Helper::instance()->config()->multitaskviewTopContentMargin() / devicePixelRatio;
     auto bottomContentMargin =
-        TreelandConfig::ref().multitaskviewBottomContentMargin() / devicePixelRatio;
-    auto cellPadding = TreelandConfig::ref().multitaskviewCellPadding() / devicePixelRatio;
+        Helper::instance()->config()->multitaskviewBottomContentMargin() / devicePixelRatio;
+    auto cellPadding = Helper::instance()->config()->multitaskviewCellPadding() / devicePixelRatio;
     auto horizontalMargin =
-        TreelandConfig::ref().multitaskviewHorizontalMargin() / devicePixelRatio;
+        Helper::instance()->config()->multitaskviewHorizontalMargin() / devicePixelRatio;
     auto availWidth = std::max(0.0, layoutArea().width() - 2 * horizontalMargin);
     auto availHeight =
         std::max(0.0, layoutArea().height() - topContentMargin - bottomContentMargin);
@@ -375,9 +380,9 @@ void MultitaskviewSurfaceModel::doCalculateLayout(const QList<ModelDataPtr> &raw
     auto devicePixelRatio = output()->outputItem()->devicePixelRatio();
     auto maxWindowHeight =
         std::min(layoutArea().height(),
-                 static_cast<qreal>(TreelandConfig::ref().normalWindowHeight() / devicePixelRatio));
-    auto minWindowHeight = TreelandConfig::ref().minMultitaskviewSurfaceHeight() / devicePixelRatio;
-    auto windowHeightStep = TreelandConfig::ref().windowHeightStep() / devicePixelRatio;
+                 static_cast<qreal>(Helper::instance()->config()->normalWindowHeight() / devicePixelRatio));
+    auto minWindowHeight = Helper::instance()->config()->minMultitaskviewSurfaceHeight() / devicePixelRatio;
+    auto windowHeightStep = Helper::instance()->config()->windowHeightStep() / devicePixelRatio;
     auto rowH = maxWindowHeight;
     while (rowH > minWindowHeight) {
         if (tryLayout(rawData, rowH)) {
@@ -409,7 +414,7 @@ void MultitaskviewSurfaceModel::doUpdateZOrder(const QList<ModelDataPtr> &rawDat
 }
 
 std::pair<int, int> MultitaskviewSurfaceModel::commitAndGetUpdateRange(
-    const QList<ModelDataPtr> &rawData)
+    [[maybe_unused]] const QList<ModelDataPtr> &rawData)
 {
     // TODO: better algorithm
     int beginIndex = 0, endIndex = -1;
@@ -692,7 +697,7 @@ void MultitaskviewSurfaceModel::setWorkspace(WorkspaceModel *newWorkspace)
     if (m_workspace)
         connectWorkspace(m_workspace);
     initializeModel();
-    emit workspaceChanged();
+    Q_EMIT workspaceChanged();
 }
 
 qreal MultitaskviewSurfaceModel::contentHeight() const
@@ -711,7 +716,7 @@ void MultitaskviewSurfaceModel::setOutput(Output *newOutput)
         return;
     m_output = newOutput;
     initializeModel();
-    emit outputChanged();
+    Q_EMIT outputChanged();
 }
 
 uint MultitaskviewSurfaceModel::count() const
