@@ -1,6 +1,4 @@
-<?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE TS>
-<TS version="2.1" language="zh_CN">
+<?xml version="1.0" ?><!DOCTYPE TS><TS version="2.1" language="zh_CN">
 <context>
     <name>WindowSelectionGrid</name>
     <message>
@@ -17,5 +15,4 @@
         <translation>释放以删除</translation>
     </message>
 </context>
-</TS>
-
+</TS>
\ No newline at end of file
