@@ -3,10 +3,12 @@
 
 #include "surface/surfacewrapper.h"
 
-#include "config/treelandconfig.h"
+#include "seat/helper.h"
+#include "treelandconfig.hpp"
 #include "core/qmlengine.h"
 #include "output/output.h"
 #include "workspace/workspace.h"
+#include "common/treelandlogging.h"
 
 #include <winputpopupsurfaceitem.h>
 #include <wlayersurface.h>
@@ -26,8 +28,6 @@
 #define CLOSE_ANIMATION 2
 #define ALWAYSONTOPLAYER 1
 
-Q_LOGGING_CATEGORY(qLcSurfaceWrapper, "treeland.shell.surfaceWrapper")
-
 SurfaceWrapper::SurfaceWrapper(QmlEngine *qmlEngine,
                                WToplevelSurface *shellSurface,
                                Type type,
@@ -232,6 +232,7 @@ SurfaceWrapper::~SurfaceWrapper()
     Q_ASSERT(!m_container);
     Q_ASSERT(!m_parentSurface);
     Q_ASSERT(m_subSurfaces.isEmpty());
+    disconnect(nullptr, nullptr, this, nullptr);
     if (m_titleBar) {
         delete m_titleBar;
         m_titleBar = nullptr;
@@ -248,6 +249,14 @@ SurfaceWrapper::~SurfaceWrapper()
         delete m_windowAnimation;
         m_windowAnimation = nullptr;
     }
+    if (m_minimizeAnimation) {
+        delete m_minimizeAnimation;
+        m_minimizeAnimation = nullptr;
+    }
+    if (m_showDesktopAnimation) {
+        delete m_showDesktopAnimation;
+        m_showDesktopAnimation = nullptr;
+    }
     if (m_coverContent) {
         delete m_coverContent;
         m_coverContent = nullptr;
@@ -533,12 +542,8 @@ void SurfaceWrapper::setSurfaceState(State newSurfaceState)
     QRectF targetGeometry;
 
     if (newSurfaceState == State::Maximized) {
-        if (!shellSurface()->maxSize().isEmpty())
-            return;
         targetGeometry = m_maximizedGeometry;
     } else if (newSurfaceState == State::Fullscreen) {
-        if (!shellSurface()->maxSize().isEmpty())
-            return;
         targetGeometry = m_fullscreenGeometry;
     } else if (newSurfaceState == State::Normal) {
         targetGeometry = m_normalGeometry;
@@ -550,6 +555,7 @@ void SurfaceWrapper::setSurfaceState(State newSurfaceState)
         startStateChangeAnimation(newSurfaceState, targetGeometry);
     } else {
         if (m_geometryAnimation) {
+            m_geometryAnimation->disconnect(this);
             m_geometryAnimation->deleteLater();
         }
 
@@ -619,7 +625,7 @@ void SurfaceWrapper::markWrapperToRemoved()
 
     if (!isWindowAnimationRunning()) {
         deleteLater();
-    } // else delete this in Animation finish
+    } // else delete this in Animation(for window close animation) finish
 }
 
 bool SurfaceWrapper::acceptKeyboardFocus() const
@@ -651,6 +657,7 @@ void SurfaceWrapper::setNoDecoration(bool newNoDecoration)
 
     if (m_noDecoration) {
         Q_ASSERT(m_decoration);
+        m_decoration->disconnect(this);
         m_decoration->deleteLater();
         m_decoration = nullptr;
     } else {
@@ -679,6 +686,7 @@ void SurfaceWrapper::updateTitleBar()
         return;
 
     if (m_titleBar) {
+        m_titleBar->disconnect(this);
         m_titleBar->deleteLater();
         m_titleBar = nullptr;
         m_surfaceItem->setTopPadding(0);
@@ -899,6 +907,7 @@ void SurfaceWrapper::onAnimationReady()
 
     if (!resize(m_pendingGeometry.size())) {
         // abort change state if resize failed
+        m_geometryAnimation->disconnect(this);
         m_geometryAnimation->deleteLater();
         return;
     }
@@ -911,6 +920,7 @@ void SurfaceWrapper::onAnimationFinished()
 {
     setXwaylandPositionFromSurface(true);
     Q_ASSERT(m_geometryAnimation);
+    m_geometryAnimation->disconnect(this);
     m_geometryAnimation->deleteLater();
 }
 
@@ -938,6 +948,7 @@ bool SurfaceWrapper::startStateChangeAnimation(State targetState, const QRectF &
 void SurfaceWrapper::onWindowAnimationFinished()
 {
     Q_ASSERT(m_windowAnimation);
+    m_windowAnimation->disconnect(this);
     m_windowAnimation->deleteLater();
     m_windowAnimation = nullptr;
 
@@ -971,7 +982,7 @@ void SurfaceWrapper::onMappedChanged()
 {
     if (m_wrapperAboutToRemove)
         return;
-    
+
     Q_ASSERT(surface());
     bool mapped = surface()->mapped() && !m_hideByLockScreen;
     if (!m_isProxy) {
@@ -1005,6 +1016,7 @@ void SurfaceWrapper::onSocketEnabledChanged()
 void SurfaceWrapper::onMinimizeAnimationFinished()
 {
     Q_ASSERT(m_minimizeAnimation);
+    m_minimizeAnimation->disconnect(this);
     m_minimizeAnimation->deleteLater();
 }
 
@@ -1046,6 +1058,7 @@ void SurfaceWrapper::setHideByLockScreen(bool hide)
 void SurfaceWrapper::onShowDesktopAnimationFinished()
 {
     Q_ASSERT(m_showDesktopAnimation);
+    m_showDesktopAnimation->disconnect(this);
     m_showDesktopAnimation->deleteLater();
     updateVisible();
 }
@@ -1054,6 +1067,7 @@ void SurfaceWrapper::startShowDesktopAnimation(bool show)
 {
 
     if (m_showDesktopAnimation) {
+        m_showDesktopAnimation->disconnect(this);
         m_showDesktopAnimation->deleteLater();
     }
 
@@ -1080,7 +1094,7 @@ qreal SurfaceWrapper::radius() const
 
     // TODO: XdgToplevel、popup、InputPopup、XWayland(bypass、widnowtype(menu、normal、popup))
     if (radius < 1 && m_type != Type::Layer) {
-        radius = TreelandConfig::ref().windowRadius();
+        radius = Helper::instance()->config()->windowRadius();
     }
 
     return radius;
@@ -1611,7 +1625,7 @@ bool SurfaceWrapper::skipDockPreView() const
 void SurfaceWrapper::setSkipDockPreView(bool skip)
 {
     if (m_type != Type::XdgToplevel && m_type != Type::XWayland) {
-        qCWarning(qLcSurfaceWrapper) << "Only xdgtoplevel and x11 surface can `setSkipDockPreView`";
+        qCWarning(treelandSurface) << "Only xdgtoplevel and x11 surface can `setSkipDockPreView`";
         return;
     }
 
@@ -1664,11 +1678,11 @@ void SurfaceWrapper::setSurfaceRole(SurfaceRole role)
     if (role != SurfaceRole::Normal) {
         setZ(ALWAYSONTOPLAYER + static_cast<int>(role));
         for (const auto &sub : std::as_const(m_subSurfaces))
-            setZ(ALWAYSONTOPLAYER + static_cast<int>(role));
+            sub->setZ(ALWAYSONTOPLAYER + static_cast<int>(role));
     } else {
         setZ(0);
         for (const auto &sub : std::as_const(m_subSurfaces))
-            setZ(0);
+            sub->setZ(0);
     }
 
     Q_EMIT surfaceRoleChanged();
