@@ -5,12 +5,14 @@
 #define protected public
 #define private public
 #include <private/qsgrenderer_p.h>
+#include <private/qsgbatchrenderer_p.h>
 #undef protected
 #undef private
 
 #include "wrenderbuffernode_p.h"
 #include "wbufferrenderer_p.h"
 #include "wqmlhelper_p.h"
+#include "platformplugin/types.h"
 
 #include <QQuickItem>
 #include <QRunnable>
@@ -23,6 +25,7 @@
 #include <private/qsgdefaultrendercontext_p.h>
 #include <private/qsgrhisupport_p.h>
 #include <private/qquickrendercontrol_p.h>
+#include <qobjectdefs.h>
 
 #include <algorithm>
 
@@ -134,7 +137,7 @@ public:
     }
 
     static DataManagerPointer<Derive> resolve(const DataManagerPointer<Derive> &other, QQuickWindow *owner) {
-        static_assert(&Derive::metaObject);
+        static_assert(QtPrivate::HasQ_OBJECT_Macro<Derive>::Value, "Derive must have Q_OBJECT macro");
         Q_ASSERT(owner);
         if (other && other->owner() == owner)
             return other;
@@ -150,34 +153,50 @@ public:
         return static_cast<QQuickWindow*>(parent());
     }
 
-    Data *resolve(Data *data, DataKeys&&... keys) {
-        if (data && get()->check(data->data, std::forward<DataKeys>(keys)...)
-            && dataList.contains(data)) {
-            return data;
-        }
-        if (data)
-            release(data);
+    std::weak_ptr<Data> resolve(std::weak_ptr<Data> data, DataKeys&&... keys) {
+        struct TryClean {
+            TryClean(DataManager *m)
+                : manager(m) {}
+            ~TryClean() {
+                manager->tryClean();
+            }
+            DataManager *manager;
+        };
 
-        for (auto data : dataList) {
+        [[maybe_unused]] TryClean cleanJob(this);
+
+        {
+            auto d = data.lock();
+            if (d && dataList.contains(d)) {
+                if (get()->check(d->data, std::forward<DataKeys>(keys)...)) {
+                    d->released = 0;
+                    return data;
+                }
+                release(data);
+            }
+        }
+
+        for (auto data : std::as_const(dataList)) {
             if (get()->check(data->data, std::forward<DataKeys>(keys)...)) {
                 data->released = 0;
                 return data;
             }
         }
 
-        data = new Data();
-        if (data->data = get()->create(std::forward<DataKeys>(keys)...)) {
-            dataList.append(data);
-            return data;
+        auto newData = std::shared_ptr<Data>(new Data());
+        if ((newData->data = get()->create(std::forward<DataKeys>(keys)...))) {
+            dataList.append(newData);
+            return newData;
         }
 
-        delete data;
-        return nullptr;
+        return {};
     }
 
-    inline void release(Data *data) {
-        data->released++;
-        ensureCleanJob();
+    inline void release(std::weak_ptr<Data> data) {
+        auto d = data.lock();
+        if (!d)
+            return;
+        d->released++;
     }
 
 protected:
@@ -191,21 +210,18 @@ protected:
 
             manager->cleanJob = nullptr;
 
-            QList<Data*> tmp;
+            QList<std::shared_ptr<Data>> tmp;
             std::swap(manager->dataList, tmp);
             manager->dataList.reserve(tmp.size());
 
-            for (Data *data : tmp) {
+            for (const auto &data : std::as_const(tmp)) {
                 if (data->released > 2) {
                     manager->get()->destroy(data->data);
-                    delete data;
                 } else {
                     manager->dataList << data;
 
-                    if (data->released > 0) {
-                        data->released++;
-                        manager->ensureCleanJob();
-                    }
+                    if (data->released > 0)
+                        ++data->released;
                 }
             }
         }
@@ -213,8 +229,8 @@ protected:
         QPointer<DataManager> manager;
     };
 
-    inline void ensureCleanJob() {
-        if (!cleanJob) {
+    inline void tryClean() {
+        if (Q_LIKELY(!cleanJob)) {
             cleanJob = new CleanJob(this);
             owner()->scheduleRenderJob(cleanJob, QQuickWindow::AfterRenderingStage);
         }
@@ -235,13 +251,12 @@ protected:
 
     using QObject::deleteLater;
     ~DataManager() {
-        for (auto data : dataList) {
+        for (auto data : std::as_const(dataList)) {
             Derive::destroy(data->data);
-            delete data;
         }
     }
 
-    QList<Data*> dataList;
+    QList<std::shared_ptr<Data>> dataList;
     QRunnable *cleanJob = nullptr;
 };
 
@@ -288,16 +303,24 @@ public:
     }
 
     void sync(const QSize &pixelSize, QSGRootNode *rootNode,
-              const QMatrix4x4 &matrix = {}, QSGRenderer *base = nullptr,
-              const QVector2D &dpr = {}) {
+              const QMatrix4x4 &matrix = {}, const QMatrix4x4 &baseProjectionMatrix = {},
+              QSGRenderer *base = nullptr, const QVector2D &dpr = {}) {
         Q_ASSERT(!renderer->rootNode());
 
         if (base) {
             renderer->setDevicePixelRatio(base->devicePixelRatio());
             renderer->setDeviceRect(base->deviceRect());
             renderer->setViewportRect(base->viewportRect());
-            renderer->setProjectionMatrix(base->projectionMatrix());
+
+            // The m22 and m23 is control the z-order for depth test.
+            // If have a base QSGRenderer, we should inherit the depth test from
+            // baseProjectionMatrix.
+            renderer->setProjectionMatrix(baseProjectionMatrix);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+            renderer->setProjectionMatrixWithNativeNDC(base->projectionMatrixWithNativeNDC(0));
+#else
             renderer->setProjectionMatrixWithNativeNDC(base->projectionMatrixWithNativeNDC());
+#endif
         } else {
             renderer->setDevicePixelRatio(1.0);
             renderer->setDeviceRect(QRect(QPoint(0, 0), pixelSize));
@@ -311,8 +334,13 @@ public:
         }
 
         if (Q_UNLIKELY(!matrix.isIdentity())) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+            renderer->setProjectionMatrix(renderer->projectionMatrix(0) * matrix);
+            renderer->setProjectionMatrixWithNativeNDC(renderer->projectionMatrixWithNativeNDC(0) * matrix);
+#else
             renderer->setProjectionMatrix(renderer->projectionMatrix() * matrix);
             renderer->setProjectionMatrixWithNativeNDC(renderer->projectionMatrixWithNativeNDC() * matrix);
+#endif
         }
 
         renderer->setRootNode(rootNode);
@@ -336,28 +364,53 @@ public:
         return true;
     }
 
-    bool render(qreal oldDPR, QRhiCommandBuffer* &oldCB) {
+    bool render(qreal oldDPR, QRhiCommandBuffer* &oldCB, bool forceDepthTest = false) {
         Q_ASSERT(renderer->m_is_rendering);
         renderer->render();
         renderer->m_is_rendering = false;
         renderer->m_changed_emitted = false;
 
         context->prepareSync(oldDPR, oldCB, graphicsConfiguration());
+        bool ok = false;
+
+        do {
+            if (forceDepthTest && Q_LIKELY(isBatchRenderer)) {
+                auto batchRenderer = static_cast<QSGBatchRenderer::Renderer*>(renderer);
+                if (auto sm = batchRenderer->m_shaderManager) {
+                    if (Q_LIKELY(!sm->pipelineCache.isEmpty())) {
+                        QVector<std::pair<QRhiGraphicsPipeline*, bool>> tmp;
+                        tmp.reserve(sm->pipelineCache.size());
+
+                        for (auto pipeline : std::as_const(sm->pipelineCache)) {
+                            tmp.append({pipeline, pipeline->hasDepthTest()});
+                            pipeline->setDepthTest(true);
+                        }
+                        ok = rhi()->endOffscreenFrame() == QRhi::FrameOpSuccess;
+                        Q_ASSERT(tmp.size() == sm->pipelineCache.size());
+                        for (auto i : std::as_const(tmp))
+                            i.first->setDepthTest(i.second);
+
+                        break;
+                    }
+                }
+            }
+
+            ok = rhi()->endOffscreenFrame() == QRhi::FrameOpSuccess;
+        } while (false);
 
-        bool ok = rhi()->endOffscreenFrame() == QRhi::FrameOpSuccess;
         renderer->setRootNode(nullptr);
 
         return ok;
     }
 
-    inline bool render(QRhiRenderTarget *rt) {
+    inline bool render(QRhiRenderTarget *rt, bool forceDepthTest = false) {
         qreal oldDPR;
         QRhiCommandBuffer *oldCB;
 
         if (!preprocess(rt, oldDPR, oldCB))
             return false;
 
-        return render(oldDPR, oldCB);
+        return render(oldDPR, oldCB, forceDepthTest);
     }
 
 private:
@@ -366,7 +419,10 @@ private:
     RhiManager(QQuickWindow *owner)
         : DataManager<RhiManager, void>(owner) {
         Q_ASSERT(owner->findChildren<RhiManager*>(Qt::FindDirectChildrenOnly).size() == 1);
-        auto rhi = QSGRhiSupport::instance()->createRhi(owner, owner);
+        std::unique_ptr<QOffscreenSurface> fallbackSurface(new QW::OffscreenSurface(nullptr));
+        fallbackSurface->create();
+
+        auto rhi = QSGRhiSupport::instance()->createRhi(owner, fallbackSurface.get());
         if (!rhi.rhi)
             return;
         Q_ASSERT(rhi.rhi->backend() == owner->rhi()->backend());
@@ -374,9 +430,16 @@ private:
         m_rhi->rhi = rhi.rhi;
         m_rhi->own = rhi.own;
         m_rhi->gc = owner->graphicsConfiguration();
+        m_rhi->offscreenSurface = fallbackSurface.release();
 
         context = QQuickWindowPrivate::get(owner)->context;
+        // Don't use RenderMode2D, when use this renderer on an exists renderTarget
+        // we need to ensure the renderer don't overwrite the depth buffer, because
+        // the exists renderTarget is using in the other QSGRenderer, maybe that
+        // renderer is using depth test, and the other QSGRenderer is not finished render.
+        // For an example: RhiNode to render its content nodes on an exists renderTarget.
         renderer = context->createRenderer(QSGRendererInterface::RenderMode2DNoDepthBuffer);
+        isBatchRenderer = dynamic_cast<QSGBatchRenderer::Renderer*>(renderer);
     }
 
     ~RhiManager() {
@@ -399,23 +462,27 @@ private:
 
     struct Rhi {
         QRhi *rhi;
+        QOffscreenSurface *offscreenSurface;
         bool own;
         QQuickGraphicsConfiguration gc;
 
-        static inline void cleanup(Rhi *pointer) {
-            if (!pointer || !pointer->own)
+        ~Rhi() {
+            if (!own)
                 return;
 
             auto rhiSupport = QSGRhiSupport::instance();
             if (rhiSupport)
-                rhiSupport->destroyRhi(pointer->rhi, pointer->gc);
+                rhiSupport->destroyRhi(rhi, gc);
             else
-                delete pointer->rhi;
+                delete rhi;
+
+            delete offscreenSurface;
         }
     };
 
     QSGRenderContext *context;
     QSGRenderer *renderer;
+    bool isBatchRenderer = false;
 
     QScopedPointer<Rhi> m_rhi;
 };
@@ -492,6 +559,17 @@ public:
         return {0};
     }
 
+    // ###: Should disable DepthAwareRendering here, because
+    // if enable depth test, Qt will render the opaque nodes first,
+    // maybe an opaque node's z-order is higher than the render node,
+    // will get the wrong texture when copy texture in RhiNode::render.
+    // But the opaque node will be cover the wrong area to the
+    // RhiNode::renderTarget after RhiNode::render, but if you using
+    // MultiEffect's blur effect for the copied texture, the blur result
+    // will contains the opaque node's contains. Fortunately, this kind
+    // of problem is not very obvious.
+    //
+    // We should fix this bug in Qt in the future.
     RenderingFlags flags() const override {
         if (Q_UNLIKELY(!contentNode))
             return BoundedRectRendering | DepthAwareRendering;
@@ -549,8 +627,9 @@ public:
 
         if (oldManager != manager) {
             sgTexture()->setTexture(nullptr);
-            if (oldManager && texture)
+            if (oldManager)
                 oldManager->release(texture);
+            texture.reset();
         }
 
         Q_ASSERT(ct->rhi() == window->rhi());
@@ -592,16 +671,18 @@ public:
         }
 
         texture = manager->resolve(texture, ct->format(), pixelSize);
-        if (Q_UNLIKELY(!texture)) {
+        if (Q_UNLIKELY(texture.expired())) {
             reset();
             return;
         }
+        auto texture = this->texture.lock();
         Q_ASSERT(texture->data);
 
         if (renderData) {
             if (!renderData->rt || sgTexture()->rhiTexture() != texture->data) {
                 QRhiTextureRenderTargetDescription rtDesc(texture->data);
-                const auto flags = QRhiTextureRenderTarget::PreserveColorContents;
+                const auto flags = QRhiTextureRenderTarget::PreserveColorContents
+                    | QRhiTextureRenderTarget::PreserveDepthStencilContents;
                 auto newRT = rhi->rhi()->newTextureRenderTarget(rtDesc, flags);
                 newRT->setRenderPassDescriptor(newRT->newCompatibleRenderPassDescriptor());
                 if (!newRT->create()) {
@@ -621,9 +702,9 @@ public:
         }
     }
 
-    void render(const RenderState *state) override {
-        Q_UNUSED(state)
+    void render([[maybe_unused]] const RenderState *state) override {
 
+        auto texture = this->texture.lock();
         if (Q_UNLIKELY(!texture))
             return;
 
@@ -637,7 +718,7 @@ public:
             const QPointF sourcePos = renderMatrix.map(m_rect.topLeft());
             renderData->imageNode->setRect(QRectF(-(devicePixelRatio - 1) * sourcePos, ct->pixelSize()));
 
-            rhi->sync(texture->data->pixelSize(), &renderData->rootNode, renderMatrix.inverted(), nullptr,
+            rhi->sync(texture->data->pixelSize(), &renderData->rootNode, renderMatrix.inverted(), {}, nullptr,
                       {texture->data->pixelSize().width() / float(m_rect.width() * devicePixelRatio),
                        texture->data->pixelSize().height() / float(m_rect.height() * devicePixelRatio)});
             rhi->render(renderData->rt.get());
@@ -669,8 +750,13 @@ public:
             Q_ASSERT(renderTarget()->resourceType() == QRhiResource::TextureRenderTarget);
             auto textureRT = static_cast<QRhiTextureRenderTarget*>(renderTarget());
             auto saveFlags = textureRT->flags();
-            textureRT->setFlags(QRhiTextureRenderTarget::PreserveColorContents);
+            textureRT->setFlags(QRhiTextureRenderTarget::PreserveColorContents
+                                | QRhiTextureRenderTarget::PreserveDepthStencilContents);
             auto currentRenderer = maybeBufferRenderer();
+            // If the source renderer enable depth test, we should enable depth test also,
+            // to ensure the contentNode's z order on RenderMode2DNoDepthBuffer render mode.
+            const bool forceDepthTest = currentRenderer && currentRenderer->currentBatchRenderer()
+                                   && currentRenderer->currentBatchRenderer()->useDepthBuffer();
 
             if (clipList() || inheritedOpacity() < 1.0) {
                 if (!node)
@@ -700,7 +786,7 @@ public:
 
                 overrideChildNodesTo(contentNode, childContainer);
 
-                rhi->sync(ct->pixelSize(), &node->rootNode, {},
+                rhi->sync(ct->pixelSize(), &node->rootNode, {}, *projectionMatrix(),
                           currentRenderer ? currentRenderer->currentRenderer() : nullptr);
                 qreal oldDPR;
                 QRhiCommandBuffer *oldCB;
@@ -717,16 +803,15 @@ public:
                         }
                     }
 
-                    rhi->render(oldDPR, oldCB);
+                    rhi->render(oldDPR, oldCB, forceDepthTest);
                 }
 
                 restoreChildNodesTo(childContainer, contentNode);
             } else {
                 node.reset();
-
-                rhi->sync(ct->pixelSize(), contentNode, *this->matrix(),
+                rhi->sync(ct->pixelSize(), contentNode, *this->matrix(), *projectionMatrix(),
                           currentRenderer ? currentRenderer->currentRenderer() : nullptr);
-                rhi->render(textureRT);
+                rhi->render(textureRT, forceDepthTest);
             }
 
             textureRT->setFlags(saveFlags);
@@ -741,9 +826,9 @@ private:
         if (!sgTexture()->rhiTexture() && notifyTexture)
             doNotifyTextureChanged();
         sgTexture()->setTexture(nullptr);
-        if (texture && manager)
-            manager->release(texture);
-        texture = nullptr;
+        if (!texture.expired() && manager)
+            manager->release(texture.lock());
+        texture.reset();
     }
 
     void destroy() {
@@ -751,11 +836,11 @@ private:
         renderData.reset();
         node.reset();
         manager = nullptr;
-        texture = nullptr;
+        texture.reset();
     }
 
     DataManagerPointer<RhiTextureManager> manager;
-    RhiTextureManager::Data *texture = nullptr;
+    std::weak_ptr<RhiTextureManager::Data> texture;
     DataManagerPointer<RhiManager> rhi;
     QMatrix4x4 renderMatrix;
     qreal devicePixelRatio;
@@ -827,7 +912,7 @@ private:
             return true;
         }
 
-        bool hasMipmaps() const {
+        bool hasMipmaps() const override {
             return mipmapFiltering() != QSGTexture::None;
         }
 
@@ -895,11 +980,10 @@ public:
 
     QImage toImage() const override
     {
-        return image ? *image->data : QImage();
+        return image.expired() ? QImage() : *image.lock()->data;
     }
 
-    void render(const RenderState *state) override {
-        Q_UNUSED(state)
+    void render([[maybe_unused]] const RenderState *state) override {
         auto window = renderWindow();
         if (!window)
             return;
@@ -917,8 +1001,9 @@ public:
 
         if (oldManager != manager) {
             texture()->setTexture(nullptr);
-            if (oldManager && image)
+            if (oldManager)
                 oldManager->release(image);
+            image.reset();
         }
 
         const bool hasRotation = matrix.flags().testAnyFlags(QMatrix4x4::Rotation2D | QMatrix4x4::Rotation);
@@ -960,6 +1045,7 @@ public:
             image = manager->resolve(image, sourceImage.format(), pixelSize);
         }
 
+        auto image = this->image.lock();
         painter.begin(image->data);
         painter.setRenderHint(QPainter::SmoothPixmapTransform);
         painter.setCompositionMode(QPainter::CompositionMode_Source);
@@ -993,20 +1079,19 @@ private:
         if (!texture()->image().isNull() && notifyTexture)
             doNotifyTextureChanged();
         texture()->setTexture(nullptr);
-        if (image && manager)
+        if (manager)
             manager->release(image);
-        image = nullptr;
+        image.reset();
     }
 
     void destroy() {
         reset(false);
         manager = nullptr;
-        image = nullptr;
     }
 
     friend class WRenderBufferNode;
     DataManagerPointer<QImageManager> manager;
-    QImageManager::Data *image = nullptr;
+    std::weak_ptr<QImageManager::Data> image;
     QPainter painter;
 };
 
@@ -1066,7 +1151,7 @@ qreal WRenderBufferNode::effectiveDevicePixelRatio() const
     if (!sgRenderer || sgRenderer->renderTarget().rt != renderTarget())
         return window->effectiveDevicePixelRatio();
 
-    return sgRenderer->devicePixelRatio();
+    return renderer->currentDevicePixelRatio();
 }
 
 WRenderBufferNode::WRenderBufferNode(QQuickItem *item, QSGTexture *texture)
