@@ -19,7 +19,7 @@ void CustomWindow::setParent(QWindow *newParent)
     QQuickWindow::setParent(newParent);
     show();
 
-    emit parentChanged();
+    Q_EMIT parentChanged();
 }
 
 int main(int argc, char *argv[])
@@ -29,11 +29,7 @@ int main(int argc, char *argv[])
     QGuiApplication app(argc, argv);
 
     QQmlApplicationEngine engine;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
-    const QUrl url(u"qrc:/qt/qml/subsurface/Main.qml"_qs);
-#else
-    const QUrl url(u"qrc:/subsurface/Main.qml"_qs);
-#endif
+    const QUrl url(QStringLiteral(u"qrc:/qt/qml/subsurface/Main.qml"));
     QObject::connect(&engine, &QQmlApplicationEngine::objectCreationFailed,
         &app, []() { QCoreApplication::exit(-1); },
         Qt::QueuedConnection);
